/*
	File:		Pipes.h

	Contains:	Pipe classes.

	Written by:	Newton Research Group.
*/

#if !defined(__PIPES_H)
#define __PIPES_H 1

DeclareException(exPipe, exRootException);


/*------------------------------------------------------------------------------
	C P i p e C a l l b a c k
------------------------------------------------------------------------------*/

class CPipeCallback
{
public:
				CPipeCallback()	{ f04 = -1; f08 = -1; }
/*virtual*/	~CPipeCallback();

//	vt			f00;
	long		f04;
	long		f08;
};


/*------------------------------------------------------------------------------
	C P i p e
------------------------------------------------------------------------------*/

class CPipe
{
public:
				CPipe();
	virtual	~CPipe();

	virtual	long		readSeek(long inOffset, int inSelector) = 0;
	virtual	long		readPosition(void) const = 0;
	virtual	long		writeSeek(long inOffset, int inSelector) = 0;
	virtual	long		writePosition(void) const = 0;
	virtual	void		readChunk(void * outBuf, unsigned long & ioSize, BOOL & ioEOF) = 0;
	virtual	void		writeChunk(const void * inBuf, unsigned long inSize, BOOL inFlush) = 0;
	virtual	void		flushRead(void) = 0;
	virtual	void		flushWrite(void) = 0;
	virtual	void		reset(void) = 0;
	virtual	void		resetRead();
	virtual	void		resetWrite();
	virtual	void		overflow() = 0;
	virtual	void		underflow(long, BOOL&) = 0;

	const CPipe &	operator>>(char&);
	const CPipe &	operator>>(unsigned char&);
	const CPipe &	operator>>(short&);
	const CPipe &	operator>>(unsigned short&);
	const CPipe &	operator>>(long&);
	const CPipe &	operator>>(unsigned long&);

	const CPipe &	operator<<(char);
	const CPipe &	operator<<(unsigned char);
	const CPipe &	operator<<(short);
	const CPipe &	operator<<(unsigned short);
	const CPipe &	operator<<(long);
	const CPipe &	operator<<(unsigned long);
};


/*------------------------------------------------------------------------------
	C P t r P i p e
------------------------------------------------------------------------------*/

class CPtrPipe : public CPipe
{
public:
				CPtrPipe();
				~CPtrPipe();

	void		init(unsigned long inSize, CPipeCallback * inCallback);
	void		init(void * inPtr, unsigned long inSize, BOOL inAssumePtrOwnership, CPipeCallback * inCallback);

	long		readSeek(long inOffset, int inSelector);
	long		readPosition(void) const;
	long		writeSeek(long inOffset, int inSelector);
	long		writePosition(void) const;
	void		readChunk(void * outBuf, unsigned long & ioSize, BOOL & ioEOF);
	void		writeChunk(const void * inBuf, unsigned long inSize, BOOL inFlush);
	void		flushRead(void);
	void		flushWrite(void);
	void		reset(void);
	void		overflow();
	void		underflow(long, BOOL&);

private:
	long		seek(long inOffset, int inSelector);

	char *				fPtr;
	unsigned long		fOffset;
	unsigned long		fEnd;
	CPipeCallback *	fCallback;
	BOOL					fPtrIsOurs;
};


/*------------------------------------------------------------------------------
	C R e f P i p e
------------------------------------------------------------------------------*/

class CRefPipe : public CPtrPipe
{
public:
				CRefPipe();
				~CRefPipe();

	void		initSink(size_t inSize, RefArg inRef, CPipeCallback * inCallback);
	void		initSource(RefArg inRef, CPipeCallback * inCallback);

private:
	long					f14;
	RefStruct			fTheRef;
};


/*------------------------------------------------------------------------------
	C S t d I O P i p e
------------------------------------------------------------------------------*/

class CStdIOPipe : public CPipe
{
public:
				CStdIOPipe(const char * inFilename, const char * inMode);
				~CStdIOPipe();

	long		readSeek(long inOffset, int inSelector);
	long		readPosition(void) const;
	long		writeSeek(long inOffset, int inSelector);
	long		writePosition(void) const;
	void		readChunk(void * outBuf, unsigned long & ioSize, BOOL & ioEOF);
	void		writeChunk(const void * inBuf, unsigned long inSize, BOOL inFlush);
	void		flushRead(void);
	void		flushWrite(void);
	void		reset(void);
	void		overflow();
	void		underflow(long, BOOL&);

private:
	long		seek(long inOffset, int inSelector);
	void		flush(void);

	FILE *	fFile;
	int		fState;
};


#endif	/* __PIPES_H */
